@@ -21,10 +21,11 @@ from pyramid.httpexceptions import \
 from pyramid_controllers import \
     includeme, \
     Controller, Dispatcher, \
-    expose, index, lookup, default, fiddle, expose_defaults
+    expose, index, lookup, default, wrap, fiddle, expose_defaults
 from pyramid_controllers.decorator import PCATTR
 from pyramid_controllers.util import getVersion
 from pyramid.config import Configurator
+import six
 
 from .test_helpers import TestHelper
 
@@ -212,7 +213,7 @@ class TestDispatcher(TestHelper):
     self.assertResponse(self.send(Root(), '/res'), 200, 'res with GET')
     self.assertResponse(self.send(Root(), '/res', method='PUT'), 200, 'res with PUT (PorP)')
     self.assertResponse(self.send(Root(), '/res', method='POST'), 200, 'res with POST (PorP)')
-    # TODO: this should return a 405...
+    # TODO: this should probably return a 405...
     self.assertResponse(self.send(Root(), '/res', method='DELETE'), 404)
 
   def test_expose_on_bound_method(self):
@@ -231,6 +232,43 @@ class TestDispatcher(TestHelper):
     self.assertResponse(self.send(bar, '/zig.txt'), 404)
     self.assertResponse(self.send(zig, '/zig.txt'), 200, 'ok.path:/zig.txt')
 
+  def test_expose_urlencodedpath(self):
+    class Root(Controller):
+      @expose(name='and/or')
+      def andor(self, request):
+        return 'and/or'
+    self.assertResponse(self.send(Root(), '/and/for'), 404)
+    self.assertResponse(self.send(Root(), '/and%2for'), 404)
+    self.assertResponse(self.send(Root(), '/and%252for'), 200, 'and/or')
+
+  def test_expose_latin1(self):
+    class Root(Controller):
+      @expose(name=u'élève')
+      def student(self, request):
+        return 'student'
+    self.assertResponse(self.send(Root(), '/élève'), 200, 'student')
+    self.assertResponse(self.send(Root(), '/%c3%a9l%c3%a8ve'), 200, 'student')
+
+  def test_expose_unicode(self):
+    class Root(Controller):
+      @expose(name=u'安装说明')
+      def instructions(self, request):
+        return 'instructions'
+    self.assertResponse(self.send(Root(), '/安装说明'), 200, 'instructions')
+    self.assertResponse(self.send(Root(), '/%E5%AE%89%E8%A3%85%E8%AF%B4%E6%98%8E'), 200, 'instructions')
+
+  def test_extra_path_segments(self):
+    class Root(Controller):
+      @index
+      def index(self, request): return 'ok.index'
+      @expose
+      def method(self, request): return 'ok.method'
+    self.assertResponse(self.send(Root(), '/method'),         200, 'ok.method')
+    self.assertResponse(self.send(Root(), '/'),               200, 'ok.index')
+    self.assertResponse(self.send(Root(), '/method/foo'),     404)
+    self.assertResponse(self.send(Root(), '/method/foo/bar'), 404)
+    self.assertResponse(self.send(Root(), '/foo'),            404)
+
   #----------------------------------------------------------------------------
   # TEST @EXPOSE ALIASING
   #----------------------------------------------------------------------------
@@ -380,6 +418,49 @@ class TestDispatcher(TestHelper):
         return 'ok:base=%s,sub=%s' % (getattr(request, 'base', 'n'), getattr(request, 'sub', 'n'))
     self.assertResponse(self.send(Sub(), '/chk'), 200, 'ok:base=y,sub=y')
 
+  #----------------------------------------------------------------------------
+  # TEST @WRAP
+  #----------------------------------------------------------------------------
+
+  def test_wrap_decorator_single(self):
+    # 'request wrapping with generator to pre- and post-process request'
+    class Sub(Controller):
+      @wrap
+      def _wrap(self, request, handler):
+        request.wrapped = True
+        ret = handler(request)
+        if not isinstance(ret, six.string_types):
+          raise Exception('expected a string response')
+        return ret + ' (post)'
+      @expose
+      def data(self, request):
+        return 'ok:wrapped=%r' % (getattr(request, 'wrapped', False),)
+    class Root(Controller):
+      sub = Sub()
+    self.assertResponse(self.send(Root(), '/sub/data'), 200, 'ok:wrapped=True (post)')
+
+  def test_wrap_decorator_multiple(self):
+    # 'request wrapping with generator to pre- and post-process request'
+    class Sub(Controller):
+      @wrap
+      def _wrap(self, request, handler):
+        ret = handler(request)
+        if not isinstance(ret, six.string_types):
+          raise Exception('expected a string response')
+        return ret + '.sub'
+      @expose
+      def data(self, request):
+        return 'ok'
+    class Root(Controller):
+      @wrap
+      def _wrap(self, request, handler):
+        ret = handler(request)
+        if not isinstance(ret, six.string_types):
+          raise Exception('expected a string response')
+        return ret + '.root'
+      sub = Sub()
+    self.assertResponse(self.send(Root(), '/sub/data'), 200, 'ok.sub.root')
+
   #----------------------------------------------------------------------------
   # TEST @LOOKUP
   #----------------------------------------------------------------------------
@@ -464,7 +545,7 @@ class TestDispatcher(TestHelper):
     self.assertResponse(self.send(Root(), '/res'), 200, 'default GET')
     self.assertResponse(self.send(Root(), '/res', method='PUT'), 200, 'default PUT (PorP)')
     self.assertResponse(self.send(Root(), '/res', method='POST'), 200, 'default POST (PorP)')
-    # TODO: this should return a 405...
+    # TODO: this should probably return a 405...
     self.assertResponse(self.send(Root(), '/res', method='DELETE'), 404)
 
   #----------------------------------------------------------------------------
